/*  叫料申请单
 * @Author: Li Hanlin
 * @Date: 2023-12-21 11:10:07
 * @Last Modified by: Li Hanlin
 * @Last Modified time: 2024-01-25 11:44:17
 */

import React, { useState, useEffect, useMemo, useRef } from "react";
import DrawerPro from "@/components/DrawerPro";
import AutoTable from "@/components/AutoTable";
import PremButton from "@/components/PremButton";
import useKeepAlive from "@/components/useKeepAlive";
import getcolumns from "./columns";
import { useRequest } from "ahooks";
import { doFetch } from "@/utils/doFetch";
import { message, Button } from "antd";
import DetailPro from "@/components/DetailPro";
function User() {
  let actionRef = useRef();
  const [drawer, setdrawer] = useState({
      open: false,
    }),
    [activeTabKey, setactiveTabKey] = useState("1"),
    [factoryNo, sfn] = useState(""),
    formRef = useRef(null);

  const { run, loading, runAsync } = useRequest(doFetch, {
    manual: true,
    onSuccess: (res, params) => {
      if (res?.code == "0000") {
        message.success("操作成功！", 2);
        reload();
      }
    },
  });
  const columns = useMemo(() => {
    let defcolumn = getcolumns(setdrawer).filter(
      (it) => it.key == activeTabKey
    )[0]?.columns;
    let defpath =
      getcolumns(setdrawer).filter((it) => it.key == activeTabKey)[0]
        ?.pathconfig ?? {};

    if (activeTabKey == 1) {
      return defcolumn.concat({
        title: "操作",
        valueType: "option",
        width: 150,
        render: (text, row, _, action) => rightExtra(text, row, _, action),
      });
    } else {
      return defcolumn;
    }
  }, [activeTabKey]);

  const tabList = useMemo(() => {
    return getcolumns();
  }, [activeTabKey]);
  const pathconfig = useMemo(() => {
    let defpath =
      getcolumns(setdrawer).filter((it) => it.key == activeTabKey)[0]
        ?.pathconfig ?? {};
    return defpath;
  }, [activeTabKey]);
  const rightExtra = (text, row, _, action) => {
    return [
      (row.documentStatus === 0 || row.documentStatus === 1) && (
        <PremButton
          key="delete"
          pop={{
            title: "是否关闭该申请单?",
            okText: "确认",
            cancelText: "取消",
            onConfirm: async () => {
              await runAsync({
                url: "/ta_wms_workmanship/pmMaterialApply/closeProduce",
                params: { id: row.id },
              });
            },
          }}
          btn={{
            size: "small",
            danger: true,
          }}
        >
          关单
        </PremButton>
      ),
    ];
  };
  function reload() {
    actionRef?.current?.reload();
    setdrawer((s) => ({
      ...s,
      open: false,
    }));
  }
  const [messageApi, contextHolder] = message.useMessage();

  const getTxByFactoryNo = useRequest(
    (s) => {
      return doFetch({
        url: "/ta_wms_workmanship/pmJobMaterial/queryTxByFactoryNo",
        params: { factoryNo: s },
      });
    },
    {
      debounceWait: 1000,
      manual: true,
    }
  );

  const getInfoByStationId = useRequest(
    (s) => {
      return doFetch({
        url: "/ta_wms_workmanship/pmJobMaterial/queryInfoByStationId",
        params: { id: s },
      });
    },
    {
      debounceWait: 1000,
      manual: true,
    }
  );
  return (
    <div>
      <AutoTable
        pagetitle="叫料申请单"
        columns={columns}
        path={pathconfig?.list}
        actionRef={actionRef}
        pageextra={activeTabKey == "1" ? "add" : null}
        addconfig={{
          // access: 'sysDepartment_save',
          name: "新增",
          btn: {
            type: "primary",
            onClick: () => {
              setdrawer((s) => ({
                ...s,
                open: true,
                title: "新增",
                val: "add",
                defaultFormValue: { taskType: "1" },
                fields: [
                  {
                    title: "出库单号",
                    dataIndex: "callMaterialNo",
                    key: "callMaterialNo",
                    fieldProps: {
                      placeholder: "不填写则系统自动生成",
                    },
                  },
                  {
                    title: "单据类型",
                    dataIndex: "taskType",
                    key: "taskType",
                    valueType: "select",
                    options: [
                      {
                        value: "1",
                        label: "生产叫料单",
                      },
                      {
                        value: "2",
                        label: "返工叫料单",
                      },
                      {
                        value: "3",
                        label: "生产补叫料单",
                      },
                    ],
                    fieldProps: {
                      allowClear: false,
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "目标位置",
                    dataIndex: "targetAreaPositionId",
                    key: "targetAreaPositionId",
                    valueType: "select",
                    hideInForm: {
                      taskType: ["1", "2"],
                    },
                    options: {
                      path: "/ta_wms_auth/sysStation/queryZpSecondRow/select",
                      linkParams: {
                        taskType: "",
                      },
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "出厂编号",
                    dataIndex: "factoryNo",
                    key: "factoryNo",
                    hideInForm: {
                      taskType: ["3"],
                    },
                    fieldProps: {
                      placeholder: "输入后自动查询产品信息",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "出厂编号",
                    dataIndex: "factoryNo",
                    key: "factoryNo",
                    hideInForm: {
                      taskType: ["1", "2"],
                    },
                    fieldProps: {
                      disabled: true,
                      placeholder: "选择目标位置后查询获取",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "产品型号",
                    dataIndex: "productModel",
                    key: "productModel",
                    hideInForm: {
                      taskType: ["1", "2"],
                    },
                    fieldProps: {
                      disabled: true,
                      placeholder: "选择目标位置后查询获取",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "工单号",
                    dataIndex: "jobNo",
                    key: "jobNo",
                    hideInForm: {
                      taskType: ["1", "2"],
                    },
                    fieldProps: {
                      disabled: true,
                      placeholder: "选择目标位置后查询获取",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "产品型号",
                    dataIndex: "productModel",
                    key: "productModel",
                    hideInForm: {
                      taskType: ["3"],
                    },
                    fieldProps: {
                      disabled: true,
                      placeholder: "输入出厂编号后查询获取",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "工单号",
                    dataIndex: "jobNo",
                    key: "jobNo",
                    hideInForm: {
                      taskType: ["3"],
                    },
                    fieldProps: {
                      disabled: true,
                      placeholder: "选择出厂编号后查询获取",
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "目标位置",
                    dataIndex: "targetAreaPositionId",
                    key: "targetAreaPositionId",
                    valueType: "select",
                    hideInForm: {
                      taskType: ["3"],
                    },
                    options: {
                      path: "/ta_wms_auth/sysStation/queryZpSecondRow/select",
                      linkParams: {
                        taskType: "",
                      },
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "线圈需求",
                    dataIndex: "coilDemand",
                    key: "coilDemand",
                    valueType: "digit",
                    fieldProps: { precision: 0 },
                    hideInForm: {
                      taskType: ["2"],
                    },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                  {
                    title: "铁芯需求",
                    dataIndex: "coreDemand",
                    key: "coreDemand",
                    hideInForm: {
                      taskType: ["2", "3"],
                    },
                    valueType: "digit",
                    fieldProps: { precision: 0 },
                    formItemProps: {
                      rules: [
                        {
                          required: true,
                          message: "此项为必填项",
                        },
                      ],
                    },
                  },
                ],

                detailpath: "",
              }));
            },
          },
        }}
        tabList={tabList}
        activeTabKey={activeTabKey}
        onTabChange={(key) => {
          setactiveTabKey(key);
        }}
      />
      <DrawerPro
        fields={columns}
        detailpath={drawer.detailpath}
        params={{ id: drawer?.item?.id }}
        placement="right"
        onClose={() => {
          setdrawer((s) => ({
            ...s,
            open: false,
          }));
        }}
        onValuesChange={(c, a) => {
          console.log(c, a);
          if (Object?.keys(c).includes("taskType")) {
            formRef?.current?.setFieldsValue({
              jobNo: "",
              productModel: "",
              factoryNo: "",
            });
          }
          if (a?.taskType && a?.taskType === "3") {
            if (Object.keys(c)?.includes("targetAreaPositionId")) {
              getInfoByStationId
                ?.runAsync(c?.targetAreaPositionId)
                .then((res) => {
                  if (res?.code === "0000" && res?.data?.data) {
                    message.success("成功查询关联数据！", 2);
                    formRef?.current?.setFieldsValue({
                      factoryNo: res.data?.data?.factoryNo,
                      jobNo: res.data?.data?.productJobNo,
                      productModel: res?.data?.data?.productModel,
                    });
                  } else {
                    message.destroy();
                    message.warning("未查询到关联数据！", 2);
                    formRef?.current?.setFieldsValue({
                      jobNo: "",
                      productModel: "",
                      factoryNo: "",
                    });
                  }
                });
            }
          } else {
            if (Object.keys(c)?.includes("factoryNo")) {
              getTxByFactoryNo?.runAsync(c?.factoryNo).then((res) => {
                if (res?.code === "0000" && res?.data?.data) {
                  message.success("成功查询关联数据！", 2);
                  formRef?.current?.setFieldsValue({
                    jobNo: res.data?.data?.productJobNo,
                    productModel: res?.data?.data?.productModel,
                  });
                } else {
                  message.destroy();
                  message.warning("未查询到关联数据！", 2);
                  formRef?.current?.setFieldsValue({
                    jobNo: "",
                    productModel: "",
                  });
                }
              });
            }
          }
        }}
        onFinish={async (vals) => {
          let newfields = JSON.parse(JSON.stringify(vals));
          console.log(newfields);
          if (newfields?.taskType === "2") {
            newfields.coilDemand = 0;
            newfields.coreDemand = 0;
          } else if (newfields?.taskType === "3") {
            newfields.coreDemand = 0;
          }
          await runAsync({
            url: "/ta_wms_workmanship/pmMaterialApply/saveProduce",
            params: {
              ...newfields,
            },
          });
        }}
        formRef={formRef}
        {...drawer}
      >
        <AutoTable
          withCard={false}
          columns={[
            {
              title: "物料编号-名称",
              dataIndex: "materialNo",
              key: "materialNo",
              render: (_, row) => {
                return (
                  <span>
                    {row?.materialNo}-{row?.materialName}
                  </span>
                );
              },
            },
            {
              title: "出库数量",
              dataIndex: "storeNum",
              key: "storeNum",
            },
            {
              title: "库存单位",
              dataIndex: "productionUnitName",
              key: "productionUnitName",
            },
            {
              title: "未下架数量",
              dataIndex: "noStoreNum",
              key: "noStoreNum",
            },
          ]}
          expandable={{
            expandedRowRender: (record) => (
              <AutoTable
                withCard={false}
                pagination={false}
                className="expandableTable"
                columns={[
                  {
                    title: "库位名称",
                    dataIndex: "storeName",
                    key: "storeName",
                  },
                  {
                    title: "批次号",
                    dataIndex: "jobNo",
                    key: "jobNo",
                  },
                  {
                    title: "供应商编号-名称",
                    dataIndex: "supplierName",
                    key: "supplierName",
                    render: (_, row) => {
                      return (
                        <span>
                          {row?.supplierNo}-{row?.supplierName}
                        </span>
                      );
                    },
                  },
                  {
                    title: "计划数量",
                    dataIndex: "planStoreNum",
                    key: "planStoreNum",
                  },
                  {
                    title: "下架数量",
                    dataIndex: "storeNum",
                    key: "storeNum",
                  },
                  {
                    title: "状态",
                    dataIndex: "statusName",
                    key: "statusName",
                  },
                ]}
                dataSource={record?.taskMaterialOutstoreDetails ?? []}
              />
            ),
            rowExpandable: (record) => record.name !== "Not Expandable",
          }}
          dataSource={drawer?.detailData?.taskMaterialDetails ?? []}
          actionRef={actionRef}
          pagination={false}
        />
        <DetailPro
          fields={[
            { title: "强制关单信息", valueType: "split" },
            { title: "关单人员", dataIndex: "closeUserName" },
            { title: "关单时间", dataIndex: "closeTime" },
          ]}
          detailData={drawer?.detailData}
        />
      </DrawerPro>
    </div>
  );
}

export default useKeepAlive(User);
